@@ -93,9 +93,11 @@ class FrameworkPipeline(Transformer):
 
             self.progress_logger.log_event(
                 event_name=pipeline_name,
-                event_text=f"Starting Pipeline {pipeline_name}" + f"_{self._run_id}"
-                if self._run_id
-                else "",
+                event_text=(
+                    f"Starting Pipeline {pipeline_name}" + f"_{self._run_id}"
+                    if self._run_id
+                    else ""
+                ),
                 log_level=LogLevel.INFO,
             )
             self.progress_logger.log_params(params=self.__parameters)
@@ -105,6 +107,9 @@ class FrameworkPipeline(Transformer):
                 if hasattr(transformer, "getName"):
                     # noinspection Mypy
                     stage_name = transformer.getName()
+                    # check that there is a value in case it is set to empty string
+                    if not stage_name:
+                        stage_name = transformer.__class__.__name__
                 else:
                     stage_name = transformer.__class__.__name__
                 try:
@@ -184,10 +189,12 @@ class FrameworkPipeline(Transformer):
             self._check_validation(df)
 
     def _check_validation(self, df: DataFrame) -> None:
-        tables_df = df.sql_ctx.tables().filter(
-            f"tableName ='{pipeline_validation_df_name}'"
-        )
-        if tables_df.count() == 1 and self.validation_output_path:
+        tables = [
+            t.name
+            for t in df.sparkSession.catalog.listTables()
+            if t.name == pipeline_validation_df_name
+        ]
+        if len(tables) == 1 and self.validation_output_path:
             FrameworkCsvExporter(
                 view=pipeline_validation_df_name,
                 file_path=self.validation_output_path,
@@ -195,7 +202,7 @@ class FrameworkPipeline(Transformer):
                 parameters=self.parameters,
                 progress_logger=self.progress_logger,
             ).transform(df)
-            errors_df = df.sql_ctx.sql(
+            errors_df = df.sparkSession.sql(
                 f"SELECT * from {pipeline_validation_df_name} where is_failed == 1"
             )
             error_count = errors_df.count()
