@@ -9,6 +9,7 @@ from typing import Any, Dict, List, Optional, Union, Callable, cast
 # noinspection PyPep8Naming
 import pyspark.sql.functions as F
 from helix_fhir_client_sdk.filters.sort_field import SortField
+from helix_fhir_client_sdk.function_types import RefreshTokenFunction
 from pyspark import StorageLevel
 from pyspark.ml.param import Param
 from pyspark.rdd import RDD
@@ -68,6 +69,7 @@ class FhirReceiver(FrameworkTransformer):
         filter_by_resource: Optional[str] = None,
         filter_parameter: Optional[str] = None,
         additional_parameters: Optional[List[str]] = None,
+        additional_parameters_view: Optional[str] = None,
         limit: Optional[int] = None,
         action: Optional[str] = None,
         action_payload: Optional[Dict[str, Any]] = None,
@@ -87,6 +89,7 @@ class FhirReceiver(FrameworkTransformer):
         expand_fhir_bundle: bool = True,
         accept_type: Optional[str] = None,
         content_type: Optional[str] = None,
+        additional_request_headers: Optional[Dict[str, str]] = None,
         accept_encoding: Optional[str] = None,
         mode: str = FileWriteModes.MODE_OVERWRITE,
         ignore_status_codes: Optional[List[int]] = None,
@@ -107,6 +110,7 @@ class FhirReceiver(FrameworkTransformer):
         cache_storage_level: Optional[StorageLevel] = None,
         graph_json: Optional[Dict[str, Any]] = None,
         run_synchronously: Optional[bool] = None,
+        refresh_token_function: Optional[RefreshTokenFunction] = None,
     ) -> None:
         """
         Transformer to call and receive FHIR resources from a FHIR server
@@ -127,6 +131,7 @@ class FhirReceiver(FrameworkTransformer):
         :param filter_parameter: Instead of requesting ?patient=1,
                 do ?subject:Patient=1 (if filter_parameter is subject)
         :param additional_parameters: Any additional parameters to send with request
+        :param additional_parameters_view: (Optional) view name for additional parameters, e.g. search parameters
         :param auth_scopes: list of scopes to request permission for e.g., system/AllergyIntolerance.read
         :param limit: maximum number of resources to get
         :param action: (Optional) do an action e.g., $everything
@@ -142,6 +147,8 @@ class FhirReceiver(FrameworkTransformer):
         :param error_on_result_count: Throw exception when the response from FHIR count does not match the request count
         :param accept_type: (Optional) Accept header to use
         :param content_type: (Optional) Content-Type header to use
+        :param additional_request_headers: (Optional) Additional request headers to use
+                                            (Eg: {"Accept-Charset": "utf-8"})
         :param accept_encoding: (Optional) Accept-encoding header to use
         :param ignore_status_codes: (Optional) do not throw an exception for these HTTP status codes
         :param mode: if output files exist, should we overwrite or append
@@ -156,6 +163,7 @@ class FhirReceiver(FrameworkTransformer):
                                     https://sparkbyexamples.com/spark/spark-dataframe-cache-and-persist-explained/.
         :param graph_json: (Optional) a FHIR GraphDefinition resource to use for retrieving data
         :param run_synchronously: (Optional) Run on the Spark master to make debugging easier on dev machines
+        :param refresh_token_function: (Optional) function to refresh the token
         """
         super().__init__(
             name=name, parameters=parameters, progress_logger=progress_logger
@@ -210,6 +218,11 @@ class FhirReceiver(FrameworkTransformer):
         )
         self._setDefault(additional_parameters=None)
 
+        self.additional_parameters_view: Param[Optional[str]] = Param(
+            self, "additional_parameters_view", ""
+        )
+        self._setDefault(additional_parameters_view=None)
+
         self.id_view: Param[Optional[str]] = Param(self, "id_view", "")
         self._setDefault(id_view=None)
 
@@ -292,6 +305,11 @@ class FhirReceiver(FrameworkTransformer):
         self.content_type: Param[Optional[str]] = Param(self, "content_type", "")
         self._setDefault(content_type=content_type)
 
+        self.additional_request_headers: Param[Optional[Dict[str, str]]] = Param(
+            self, "additional_request_headers", ""
+        )
+        self._setDefault(additional_request_headers=additional_request_headers)
+
         self.accept_encoding: Param[Optional[str]] = Param(self, "accept_encoding", "")
         self._setDefault(accept_encoding=accept_encoding)
 
@@ -367,19 +385,28 @@ class FhirReceiver(FrameworkTransformer):
         )
         self._setDefault(run_synchronously=run_synchronously)
 
+        self.refresh_token_function: Param[Optional[RefreshTokenFunction]] = Param(
+            self, "refresh_token_function", ""
+        )
+        self._setDefault(refresh_token_function=refresh_token_function)
+
         kwargs = self._input_kwargs
         self.setParams(**kwargs)
 
     def _transform(self, df: DataFrame) -> DataFrame:
         server_url: Optional[str] = self.getServerUrl()
         resource_name: str = self.getResource()
+        parameters = self.getParameters()
         filter_by_resource: Optional[str] = self.getFilterByResource()
         filter_parameter: Optional[str] = self.getFilterParameter()
         additional_parameters: Optional[List[str]] = self.getAdditionalParameters()
+        additional_parameters_view: Optional[str] = self.getOrDefault(
+            self.additional_parameters_view
+        )
         id_view: Optional[str] = self.getIdView()
-        file_path: Union[
-            Path, str, Callable[[Optional[str]], Union[Path, str]]
-        ] = self.getFilePath()
+        file_path: Union[Path, str, Callable[[Optional[str]], Union[Path, str]]] = (
+            self.getFilePath()
+        )
         if callable(file_path):
             file_path = file_path(self.loop_id)
         name: Optional[str] = self.getName()
@@ -410,6 +437,9 @@ class FhirReceiver(FrameworkTransformer):
         accept_type: Optional[str] = self.getAcceptType()
         content_type: Optional[str] = self.getContentType()
         accept_encoding: Optional[str] = self.getAcceptEncoding()
+        additional_request_headers: Optional[Dict[str, str]] = (
+            self.getAdditionalRequestHeaders()
+        )
 
         ignore_status_codes: List[int] = self.getIgnoreStatusCodes() or []
         ignore_status_codes.append(200)
@@ -450,6 +480,31 @@ class FhirReceiver(FrameworkTransformer):
 
         run_synchronously: Optional[bool] = self.getOrDefault(self.run_synchronously)
 
+        refresh_token_function: Optional[RefreshTokenFunction] = self.getOrDefault(
+            self.refresh_token_function
+        )
+
+        if parameters and parameters.get("flow_name"):
+            user_agent_value = (
+                f"{parameters['team_name']}:helix.pipelines:{parameters['flow_name']}".replace(
+                    " ", ""
+                )
+                if parameters.get("team_name")
+                else str(parameters["flow_name"])
+            )
+            if additional_request_headers:
+                additional_request_headers.update(
+                    {
+                        "User-Agent": user_agent_value,
+                        "Origin-Service": f"helix.pipelines:{parameters['flow_name']}",
+                    }
+                )
+            else:
+                additional_request_headers = {
+                    "User-Agent": user_agent_value,
+                    "Origin-Service": f"helix.pipelines:{parameters['flow_name']}",
+                }
+
         # get access token first so we can reuse it
         if auth_client_id and server_url:
             auth_access_token = fhir_get_access_token(
@@ -466,10 +521,25 @@ class FhirReceiver(FrameworkTransformer):
         with ProgressLogMetric(
             name=f"{name}_fhir_receiver", progress_logger=progress_logger
         ):
+            # if additional_parameters_view is given,
+            # this is for generic use. e.g. to support blocking by adding search parameters in a query to FHIR
+            if additional_parameters_view and additional_parameters:
+                # created a df for additional_parameters_view
+                # the data in this view MUST have 1 row per each parameter and only 1 column per each row.
+                # e.g. "address-postalcode=10304,11040,11801" for a comma separate postal codes for Person resource
+                df_additional_parameters_view: DataFrame = df.sparkSession.table(
+                    additional_parameters_view
+                )
+                for row in df_additional_parameters_view.collect():
+                    # get the data from each row for each parameter, e.g. "address-postalcode=10304,11040,11801"
+                    parameter_values: str = str(row[0])
+                    # add it to the additional_parameters
+                    additional_parameters += [f"{parameter_values}"]
+
             # if we're calling for individual ids
             # noinspection GrazieInspection
             if id_view:
-                id_df: DataFrame = df.sql_ctx.table(id_view)
+                id_df: DataFrame = df.sparkSession.table(id_view)
                 if spark_is_data_frame_empty(df=id_df):
                     # nothing to do
                     return df
@@ -500,42 +570,43 @@ class FhirReceiver(FrameworkTransformer):
                 result_with_counts_and_responses: DataFrame
                 if run_synchronously:
                     id_rows: List[Row] = id_df.collect()
-                    result_rows: List[
-                        Row
-                    ] = FhirReceiverHelpers.send_partition_request_to_server(
-                        partition_index=0,
-                        rows=id_rows,
-                        batch_size=batch_size,
-                        has_token_col=has_token_col,
-                        server_url=server_url,
-                        log_level=log_level,
-                        action=action,
-                        action_payload=action_payload,
-                        additional_parameters=additional_parameters,
-                        filter_by_resource=filter_by_resource,
-                        filter_parameter=filter_parameter,
-                        sort_fields=sort_fields,
-                        auth_server_url=auth_server_url,
-                        auth_client_id=auth_client_id,
-                        auth_client_secret=auth_client_secret,
-                        auth_login_token=auth_login_token,
-                        auth_scopes=auth_scopes,
-                        include_only_properties=include_only_properties,
-                        separate_bundle_resources=separate_bundle_resources,
-                        expand_fhir_bundle=expand_fhir_bundle,
-                        accept_type=accept_type,
-                        content_type=content_type,
-                        accept_encoding=accept_encoding,
-                        slug_column=slug_column,
-                        retry_count=retry_count,
-                        exclude_status_codes_from_retry=exclude_status_codes_from_retry,
-                        limit=limit,
-                        auth_access_token=auth_access_token,
-                        resource_type=resource_name,
-                        error_view=error_view,
-                        url_column=url_column,
-                        use_data_streaming=use_data_streaming,
-                        graph_json=graph_json,
+                    result_rows: List[Row] = (
+                        FhirReceiverHelpers.send_partition_request_to_server(
+                            partition_index=0,
+                            rows=id_rows,
+                            batch_size=batch_size,
+                            has_token_col=has_token_col,
+                            server_url=server_url,
+                            log_level=log_level,
+                            action=action,
+                            action_payload=action_payload,
+                            additional_parameters=additional_parameters,
+                            filter_by_resource=filter_by_resource,
+                            filter_parameter=filter_parameter,
+                            sort_fields=sort_fields,
+                            auth_server_url=auth_server_url,
+                            auth_client_id=auth_client_id,
+                            auth_client_secret=auth_client_secret,
+                            auth_login_token=auth_login_token,
+                            auth_scopes=auth_scopes,
+                            include_only_properties=include_only_properties,
+                            separate_bundle_resources=separate_bundle_resources,
+                            expand_fhir_bundle=expand_fhir_bundle,
+                            accept_type=accept_type,
+                            content_type=content_type,
+                            additional_request_headers=additional_request_headers,
+                            accept_encoding=accept_encoding,
+                            slug_column=slug_column,
+                            retry_count=retry_count,
+                            exclude_status_codes_from_retry=exclude_status_codes_from_retry,
+                            limit=limit,
+                            auth_access_token=auth_access_token,
+                            resource_type=resource_name,
+                            error_view=error_view,
+                            url_column=url_column,
+                            use_data_streaming=use_data_streaming,
+                            graph_json=graph_json,
+                        )
                     )
                     response_schema = FhirGetResponseSchema.get_schema()
 
@@ -570,6 +641,7 @@ class FhirReceiver(FrameworkTransformer):
                             expand_fhir_bundle=expand_fhir_bundle,
                             accept_type=accept_type,
                             content_type=content_type,
+                            additional_request_headers=additional_request_headers,
                             accept_encoding=accept_encoding,
                             slug_column=slug_column,
                             retry_count=retry_count,
@@ -711,13 +783,10 @@ class FhirReceiver(FrameworkTransformer):
                         if progress_logger and not spark_is_data_frame_empty(errors_df):
                             progress_logger.log_event(
                                 event_name="Errors receiving FHIR",
-                                event_text=json.dumps(
-                                    get_pretty_data_frame(
-                                        df=errors_df,
-                                        limit=100,
-                                        name="Errors Receiving FHIR",
-                                    ),
-                                    default=str,
+                                event_text=get_pretty_data_frame(
+                                    df=errors_df,
+                                    limit=100,
+                                    name="Errors Receiving FHIR",
                                 ),
                                 log_level=LogLevel.INFO,
                             )
@@ -902,6 +971,7 @@ class FhirReceiver(FrameworkTransformer):
                     expand_fhir_bundle=expand_fhir_bundle,
                     accept_type=accept_type,
                     content_type=content_type,
+                    additional_request_headers=additional_request_headers,
                     accept_encoding=accept_encoding,
                     retry_count=retry_count,
                     exclude_status_codes_from_retry=exclude_status_codes_from_retry,
@@ -910,6 +980,7 @@ class FhirReceiver(FrameworkTransformer):
                     ignore_status_codes=ignore_status_codes,
                     use_data_streaming=use_data_streaming,
                     graph_json=graph_json,
+                    refresh_token_function=refresh_token_function,
                 )
                 resources = result1.resources
                 errors = result1.errors
@@ -949,13 +1020,10 @@ class FhirReceiver(FrameworkTransformer):
                     if progress_logger and not spark_is_data_frame_empty(errors_df):
                         progress_logger.log_event(
                             event_name="Errors receiving FHIR",
-                            event_text=json.dumps(
-                                get_pretty_data_frame(
-                                    df=errors_df,
-                                    limit=100,
-                                    name="Errors Receiving FHIR",
-                                ),
-                                default=str,
+                            event_text=get_pretty_data_frame(
+                                df=errors_df,
+                                limit=100,
+                                name="Errors Receiving FHIR",
                             ),
                             log_level=LogLevel.INFO,
                         )
@@ -1068,6 +1136,10 @@ class FhirReceiver(FrameworkTransformer):
     def getContentType(self) -> Optional[str]:
         return self.getOrDefault(self.content_type)
 
+    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
+    def getAdditionalRequestHeaders(self) -> Optional[Dict[str, str]]:
+        return self.getOrDefault(self.additional_request_headers)
+
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
     def getAcceptEncoding(self) -> Optional[str]:
         return self.getOrDefault(self.accept_encoding)
