@@ -9,7 +9,7 @@ from pyspark import StorageLevel
 from pyspark.ml.param import Param
 from pyspark.rdd import RDD
 from pyspark.sql.dataframe import DataFrame
-from pyspark.sql.functions import col
+from pyspark.sql.functions import col, get_json_object
 from pyspark.sql.types import Row
 from pyspark.sql.utils import AnalysisException
 
@@ -18,7 +18,7 @@ from spark_pipeline_framework.progress_logger.progress_log_metric import (
     ProgressLogMetric,
 )
 from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger
-from spark_pipeline_framework.transformers.fhir_sender.v1.fhir_sender_operation import (
+from spark_pipeline_framework.utilities.fhir_helpers.fhir_sender_operation import (
     FhirSenderOperation,
 )
 from spark_pipeline_framework.transformers.fhir_sender.v1.fhir_sender_processor import (
@@ -45,6 +45,7 @@ from spark_pipeline_framework.utilities.flattener.flattener import flatten
 from spark_pipeline_framework.utilities.spark_data_frame_helpers import (
     spark_is_data_frame_empty,
 )
+from spark_pipeline_framework.utilities.map_functions import remove_field_from_json
 
 
 class FhirSender(FrameworkTransformer):
@@ -58,6 +59,7 @@ class FhirSender(FrameworkTransformer):
         resource: str,
         batch_size: Optional[int] = 30,
         limit: int = -1,
+        file_format: str = "json",
         validation_server_url: Optional[str] = None,
         throw_exception_on_validation_failure: Optional[bool] = None,
         auth_server_url: Optional[str] = None,
@@ -65,10 +67,12 @@ class FhirSender(FrameworkTransformer):
         auth_client_secret: Optional[str] = None,
         auth_login_token: Optional[str] = None,
         auth_scopes: Optional[List[str]] = None,
+        auth_well_known_url: Optional[str] = None,
         operation: Union[
             FhirSenderOperation, str
         ] = FhirSenderOperation.FHIR_OPERATION_MERGE.value,
         name: Optional[str] = None,
+        additional_request_headers: Optional[Dict[str, str]] = None,
         parameters: Optional[Dict[str, Any]] = None,
         progress_logger: Optional[ProgressLogger] = None,
         mode: str = FileWriteModes.MODE_OVERWRITE,
@@ -81,6 +85,10 @@ class FhirSender(FrameworkTransformer):
         delta_lake_table: Optional[str] = None,
         cache_storage_level: Optional[StorageLevel] = None,
         run_synchronously: Optional[bool] = None,
+        sort_by_column_name_and_type: Optional[tuple[str, Any]] = None,
+        drop_fields_from_json: Optional[List[str]] = None,
+        partition_by_column_name: Optional[str] = None,
+        enable_repartitioning: bool = True,
     ):
         """
         Sends FHIR json stored in a folder to a FHIR server
@@ -97,6 +105,8 @@ class FhirSender(FrameworkTransformer):
         :param auth_client_secret: client secret
         :param auth_login_token: login token to send auth server
         :param auth_scopes: scopes to request
+        :param additional_request_headers: (Optional) Additional request headers to use
+                                            (Eg: {"Accept-Charset": "utf-8"})
         :param operation: What FHIR operation to perform (e.g. $merge, delete, etc.)
         :param mode: if output files exist, should we overwrite or append
         :param error_view: (Optional) log errors into this view (view only exists IF there are errors)
@@ -107,6 +117,10 @@ class FhirSender(FrameworkTransformer):
         :param cache_storage_level: (Optional) how to store the cache:
                                     https://sparkbyexamples.com/spark/spark-dataframe-cache-and-persist-explained/.
         :param run_synchronously: (Optional) Run on the Spark master to make debugging easier on dev machines
+        :param sort_by_column_name_and_type: (Optional) tuple of columnName, columnType to be used for sorting
+        :param drop_fields_from_json: (Optional) List of field names to drop from json
+        :param partition_by_column_name: (Optional) Name of the column that will be used to repartition df
+        :param enable_repartitioning: Enable repartitioning or not, default True
         """
         super().__init__(
             name=name, parameters=parameters, progress_logger=progress_logger
@@ -155,6 +169,9 @@ class FhirSender(FrameworkTransformer):
             throw_exception_on_validation_failure=throw_exception_on_validation_failure
         )
 
+        self.file_format: Param[str] = Param(self, "file_format", "")
+        self._setDefault(file_format=file_format)
+
         self.validation_server_url: Param[Optional[str]] = Param(
             self, "validation_server_url", ""
         )
@@ -179,11 +196,21 @@ class FhirSender(FrameworkTransformer):
         self.auth_scopes: Param[List[str]] = Param(self, "auth_scopes", "")
         self._setDefault(auth_scopes=None)
 
+        self.auth_well_known_url: Param[Optional[str]] = Param(
+            self, "auth_well_known_url", ""
+        )
+        self._setDefault(auth_well_known_url=None)
+
         self.operation: Param[Union[FhirSenderOperation, str]] = Param(
             self, "operation", ""
         )
         self._setDefault(operation=operation)
 
+        self.additional_request_headers: Param[Optional[Dict[str, str]]] = Param(
+            self, "additional_request_headers", ""
+        )
+        self._setDefault(additional_request_headers=additional_request_headers)
+
         self.mode: Param[str] = Param(self, "mode", "")
         self._setDefault(mode=mode)
 
@@ -224,28 +251,52 @@ class FhirSender(FrameworkTransformer):
         )
         self._setDefault(run_synchronously=run_synchronously)
 
+        self.sort_by_column_name_and_type: Param[Optional[tuple[str, Any]]] = Param(
+            self, "sort_by_column_name_and_type", ""
+        )
+        self._setDefault(sort_by_column_name_and_type=sort_by_column_name_and_type)
+
+        self.drop_fields_from_json: Param[Optional[List[str]]] = Param(
+            self, "drop_fields_from_json", ""
+        )
+        self._setDefault(drop_fields_from_json=drop_fields_from_json)
+
+        self.partition_by_column_name: Param[Optional[str]] = Param(
+            self, "partition_by_column_name", ""
+        )
+        self._setDefault(partition_by_column_name=partition_by_column_name)
+
+        self.enable_repartitioning: Param[bool] = Param(
+            self, "enable_repartitioning", ""
+        )
+        self._setDefault(enable_repartitioning=enable_repartitioning)
+
         kwargs = self._input_kwargs
         self.setParams(**kwargs)
 
     def _transform(self, df: DataFrame) -> DataFrame:
-        file_path: Union[
-            Path, str, Callable[[Optional[str]], Union[Path, str]]
-        ] = self.getFilePath()
+        file_path: Union[Path, str, Callable[[Optional[str]], Union[Path, str]]] = (
+            self.getFilePath()
+        )
         if callable(file_path):
             file_path = file_path(self.loop_id)
-        response_path: Union[
-            Path, str, Callable[[Optional[str]], Union[Path, str]]
-        ] = self.getResponsePath()
+        response_path: Union[Path, str, Callable[[Optional[str]], Union[Path, str]]] = (
+            self.getResponsePath()
+        )
         if callable(response_path):
             response_path = response_path(self.loop_id)
         name: Optional[str] = self.getName()
         progress_logger: Optional[ProgressLogger] = self.getProgressLogger()
         resource_name: str = self.getResource()
+        parameters = self.getParameters()
+        additional_request_headers: Optional[Dict[str, str]] = (
+            self.getAdditionalRequestHeaders()
+        )
         server_url: str = self.getServerUrl()
         batch_size: Optional[int] = self.getBatchSize()
-        throw_exception_on_validation_failure: Optional[
-            bool
-        ] = self.getThrowExceptionOnValidationFailure()
+        throw_exception_on_validation_failure: Optional[bool] = (
+            self.getThrowExceptionOnValidationFailure()
+        )
         operation: Union[FhirSenderOperation, str] = self.getOrDefault(self.operation)
         mode: str = self.getMode()
 
@@ -254,14 +305,25 @@ class FhirSender(FrameworkTransformer):
         cache_storage_level: Optional[StorageLevel] = self.getOrDefault(
             self.cache_storage_level
         )
+        sort_by_column_name_and_type: Optional[tuple[str, Any]] = self.getOrDefault(
+            self.sort_by_column_name_and_type
+        )
+        drop_fields_from_json: Optional[List[str]] = self.getOrDefault(
+            self.drop_fields_from_json
+        )
+        partition_by_column_name: Optional[str] = self.getOrDefault(
+            self.partition_by_column_name
+        )
 
         if not batch_size or batch_size == 0:
             batch_size = 30
         else:
             batch_size = int(batch_size)
 
+        file_format: str = self.getFileFormat()
         validation_server_url: Optional[str] = self.getValidationServerUrl()
         auth_server_url: Optional[str] = self.getAuthServerUrl()
+        auth_well_known_url: Optional[str] = self.getOrDefault(self.auth_well_known_url)
         auth_client_id: Optional[str] = self.getAuthClientId()
         auth_client_secret: Optional[str] = self.getAuthClientSecret()
         auth_login_token: Optional[str] = self.getAuthLoginToken()
@@ -282,9 +344,34 @@ class FhirSender(FrameworkTransformer):
         )
 
         num_partitions: Optional[int] = self.getOrDefault(self.num_partitions)
+        enable_repartitioning: bool = self.getOrDefault(self.enable_repartitioning)
 
         run_synchronously: Optional[bool] = self.getOrDefault(self.run_synchronously)
 
+        if not enable_repartitioning:
+            assert (partition_by_column_name or num_partitions) is None
+
+        if parameters and parameters.get("flow_name"):
+            user_agent_value = (
+                f"{parameters['team_name']}:helix.pipelines:{parameters['flow_name']}".replace(
+                    " ", ""
+                )
+                if parameters.get("team_name")
+                else str(parameters["flow_name"])
+            )
+            if additional_request_headers:
+                additional_request_headers.update(
+                    {
+                        "User-Agent": user_agent_value,
+                        "Origin-Service": f"helix.pipelines:{parameters['flow_name']}",
+                    }
+                )
+            else:
+                additional_request_headers = {
+                    "User-Agent": user_agent_value,
+                    "Origin-Service": f"helix.pipelines:{parameters['flow_name']}",
+                }
+
         # get access token first so we can reuse it
         if auth_client_id:
             auth_access_token = fhir_get_access_token(
@@ -296,6 +383,7 @@ class FhirSender(FrameworkTransformer):
                 auth_login_token=auth_login_token,
                 auth_scopes=auth_scopes,
                 log_level=log_level,
+                auth_well_known_url=auth_well_known_url,
             )
 
         self.logger.info(
@@ -308,13 +396,18 @@ class FhirSender(FrameworkTransformer):
             path_to_files: str = str(file_path)
             try:
                 if delta_lake_table:
-                    json_df: DataFrame = df.sql_ctx.read.format("delta").load(
+                    json_df: DataFrame = df.sparkSession.read.format("delta").load(
+                        path_to_files
+                    )
+                elif file_format == "parquet":
+                    json_df = df.sparkSession.read.format(file_format).load(
                         path_to_files
                     )
                 else:
-                    json_df = df.sql_ctx.read.text(
+                    json_df = df.sparkSession.read.text(
                         path_to_files, pathGlobFilter="*.json", recursiveFileLookup=True
                     )
+
             except AnalysisException as e:
                 if str(e).startswith("Path does not exist:"):
                     if progress_logger:
@@ -329,9 +422,39 @@ class FhirSender(FrameworkTransformer):
                     f" with batch_size {batch_size}  -----"
                 )
                 assert batch_size and batch_size > 0
-                desired_partitions: int = num_partitions or math.ceil(
-                    row_count / batch_size
+                desired_partitions: int = (
+                    (num_partitions or math.ceil(row_count / batch_size))
+                    if enable_repartitioning
+                    else json_df.rdd.getNumPartitions()
                 )
+                if partition_by_column_name:
+                    json_df = json_df.withColumn(
+                        partition_by_column_name,
+                        get_json_object(col("value"), f"$.{partition_by_column_name}"),
+                    ).repartition(desired_partitions, partition_by_column_name)
+                    json_df = json_df.drop(partition_by_column_name)
+
+                if sort_by_column_name_and_type:
+                    column_name, column_type = sort_by_column_name_and_type
+                    json_df = json_df.withColumn(
+                        column_name,
+                        get_json_object(col("value"), f"$.{column_name}").cast(
+                            column_type
+                        ),
+                    ).sortWithinPartitions(column_name)
+                    json_df = json_df.drop(column_name)
+
+                if drop_fields_from_json:
+                    json_schema = json_df.schema
+                    # removing sorted field from all the json values
+                    json_df = json_df.rdd.map(
+                        lambda row: Row(
+                            value=remove_field_from_json(
+                                row["value"], drop_fields_from_json
+                            ),
+                        ),
+                    ).toDF(json_schema)
+
                 self.logger.info(
                     f"----- Total Batches for {resource_name}: {desired_partitions}  -----"
                 )
@@ -354,6 +477,7 @@ class FhirSender(FrameworkTransformer):
                             auth_login_token=auth_login_token,
                             auth_scopes=auth_scopes,
                             auth_access_token=auth_access_token,
+                            additional_request_headers=additional_request_headers,
                             log_level=log_level,
                             batch_size=batch_size,
                             validation_server_url=validation_server_url,
@@ -362,16 +486,18 @@ class FhirSender(FrameworkTransformer):
                         )
                     )
                     result_rows: List[Dict[str, Any]] = flatten(result_rows_list)
-                    result_df = df.sparkSession.createDataFrame(
+                    result_df = df.sparkSession.createDataFrame(  # type: ignore[type-var]
                         result_rows, schema=FhirMergeResponseItemSchema.get_schema()
                     )
                 else:
                     # ---- Now process all the results ----
+                    if enable_repartitioning and not partition_by_column_name:
+                        # repartition only when we haven't already repartitioned by column and
+                        # enable_repartitioning is True
+                        json_df = json_df.repartition(desired_partitions)
                     rdd: RDD[
                         Union[List[Dict[str, Any]], List[List[Dict[str, Any]]]]
-                    ] = json_df.repartition(
-                        desired_partitions
-                    ).rdd.mapPartitionsWithIndex(
+                    ] = json_df.rdd.mapPartitionsWithIndex(
                         lambda partition_index, rows: FhirSenderProcessor.send_partition_to_server(
                             partition_index=partition_index,
                             rows=rows,
@@ -386,6 +512,7 @@ class FhirSender(FrameworkTransformer):
                             auth_login_token=auth_login_token,
                             auth_scopes=auth_scopes,
                             auth_access_token=auth_access_token,
+                            additional_request_headers=additional_request_headers,
                             log_level=log_level,
                             batch_size=batch_size,
                             validation_server_url=validation_server_url,
@@ -423,12 +550,16 @@ class FhirSender(FrameworkTransformer):
                         self.logger.info(
                             f"Executing requests and writing FHIR {resource_name} responses to disk..."
                         )
-                        result_df.write.mode(mode).json(str(response_path))
+                        result_df.write.mode(mode).format(file_format).save(
+                            str(response_path)
+                        )
                         # result_df.show(truncate=False, n=100)
                         self.logger.info(
                             f"Reading from disk and counting rows for {resource_name}..."
                         )
-                        result_df = df.sql_ctx.read.json(str(response_path))
+                        result_df = df.sparkSession.read.format(file_format).load(
+                            str(response_path)
+                        )
                         file_row_count: int = result_df.count()
                         self.logger.info(
                             f"Wrote {file_row_count} FHIR {resource_name} responses to {response_path}"
@@ -520,6 +651,10 @@ class FhirSender(FrameworkTransformer):
     def getName(self) -> Optional[str]:
         return self.getOrDefault(self.name) or f" - {self.getResource()}"
 
+    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
+    def getFileFormat(self) -> str:
+        return self.getOrDefault(self.file_format)
+
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
     def getValidationServerUrl(self) -> Optional[str]:
         return self.getOrDefault(self.validation_server_url)
@@ -555,3 +690,7 @@ class FhirSender(FrameworkTransformer):
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
     def getIgnoreStatusCodes(self) -> Optional[List[int]]:
         return self.getOrDefault(self.ignore_status_codes)
+
+    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
+    def getAdditionalRequestHeaders(self) -> Optional[Dict[str, str]]:
+        return self.getOrDefault(self.additional_request_headers)
