@@ -5,7 +5,7 @@ from helix_fhir_client_sdk.responses.fhir_merge_response import FhirMergeRespons
 from pyspark.sql.types import Row
 
 from spark_pipeline_framework.logger.yarn_logger import get_logger
-from spark_pipeline_framework.transformers.fhir_sender.v1.fhir_sender_operation import (
+from spark_pipeline_framework.utilities.fhir_helpers.fhir_sender_operation import (
     FhirSenderOperation,
 )
 from spark_pipeline_framework.utilities.fhir_helpers.fhir_merge_response_item_schema import (
@@ -14,6 +14,7 @@ from spark_pipeline_framework.utilities.fhir_helpers.fhir_merge_response_item_sc
 from spark_pipeline_framework.utilities.fhir_helpers.fhir_sender_helpers import (
     send_fhir_delete,
     send_json_bundle_to_fhir,
+    update_json_bundle_to_fhir,
 )
 from spark_pipeline_framework.utilities.json_helpers import convert_dict_to_fhir_json
 
@@ -36,6 +37,7 @@ class FhirSenderProcessor:
         auth_login_token: Optional[str],
         auth_scopes: Optional[List[str]],
         auth_access_token: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         log_level: Optional[str],
         batch_size: Optional[int],
         validation_server_url: Optional[str],
@@ -65,9 +67,60 @@ class FhirSenderProcessor:
         request_id_list: List[str] = []
         responses: List[Dict[str, Any]] = []
         if FhirSenderOperation.operation_equals(
+            operation, FhirSenderOperation.FHIR_OPERATION_PATCH
+        ):
+            for item in json_data_list:
+                item_value = json.loads(item["value"])
+                payload = [
+                    json.loads(convert_dict_to_fhir_json(payload_item))
+                    for payload_item in item_value["payload"]
+                ]
+                patch_result: Optional[Dict[str, Any]] = update_json_bundle_to_fhir(
+                    obj_id=item_value["id"],
+                    json_data=json.dumps(payload),
+                    server_url=server_url,
+                    operation=operation,
+                    validation_server_url=validation_server_url,
+                    resource=resource_name,
+                    logger=logger,
+                    auth_server_url=auth_server_url,
+                    auth_client_id=auth_client_id,
+                    auth_client_secret=auth_client_secret,
+                    auth_login_token=auth_login_token,
+                    auth_scopes=auth_scopes,
+                    auth_access_token=auth_access_token,
+                    additional_request_headers=additional_request_headers,
+                    log_level=log_level,
+                )
+                if patch_result:
+                    responses.append(patch_result)
+        elif FhirSenderOperation.operation_equals(
+            operation, FhirSenderOperation.FHIR_OPERATION_PUT
+        ):
+            for item in json_data_list:
+                item_value = json.loads(item["value"])
+                put_result: Optional[Dict[str, Any]] = update_json_bundle_to_fhir(
+                    obj_id=item_value["id"],
+                    json_data=convert_dict_to_fhir_json(item_value),
+                    server_url=server_url,
+                    operation=operation,
+                    validation_server_url=validation_server_url,
+                    resource=resource_name,
+                    logger=logger,
+                    auth_server_url=auth_server_url,
+                    auth_client_id=auth_client_id,
+                    auth_client_secret=auth_client_secret,
+                    auth_login_token=auth_login_token,
+                    auth_scopes=auth_scopes,
+                    auth_access_token=auth_access_token,
+                    additional_request_headers=additional_request_headers,
+                    log_level=log_level,
+                )
+                if put_result:
+                    responses.append(put_result)
+        elif FhirSenderOperation.operation_equals(
             operation, FhirSenderOperation.FHIR_OPERATION_DELETE
         ):
-            item: Row
             # FHIR doesn't support bulk deletes, so we have to send one at a time
             responses = [
                 send_fhir_delete(
@@ -83,6 +136,7 @@ class FhirSenderProcessor:
                     auth_login_token=auth_login_token,
                     auth_scopes=auth_scopes,
                     auth_access_token=auth_access_token,
+                    additional_request_headers=additional_request_headers,
                     log_level=log_level,
                 )
                 for item in json_data_list
@@ -98,63 +152,87 @@ class FhirSenderProcessor:
                 count: int = len(json_data_list)
                 for item in json_data_list:
                     i += 1
-                    print(
-                        f"Sending {i} of {count} from partition {partition_index} "
-                        f"for {resource_name}: {json.dumps(item.asDict(recursive=True), default=str)}"
-                    )
                     current_bundle: List[Dict[str, Any]]
-                    result: Optional[FhirMergeResponse] = send_json_bundle_to_fhir(
-                        json_data_list=[
+                    json_data_list_1: List[str] = [
+                        (
                             convert_dict_to_fhir_json(item.asDict(recursive=True))
                             if "id" in item
                             else item["value"]
-                        ],
+                        )
+                    ]
+                    first_item: Optional[Dict[str, Any]] = (
+                        json.loads(json_data_list_1[0])
+                        if len(json_data_list_1) > 0
+                        else None
+                    )
+                    first_id = first_item.get("id") if first_item is not None else None
+                    print(
+                        f"Sending {i} of {count} from partition {partition_index} "
+                        f"for {resource_name}: {first_id}"
+                    )
+                    if first_id:
+                        result: Optional[FhirMergeResponse] = send_json_bundle_to_fhir(
+                            id_=first_id,
+                            json_data_list=json_data_list_1,
+                            server_url=server_url,
+                            validation_server_url=validation_server_url,
+                            resource=resource_name,
+                            logger=logger,
+                            auth_server_url=auth_server_url,
+                            auth_client_id=auth_client_id,
+                            auth_client_secret=auth_client_secret,
+                            auth_login_token=auth_login_token,
+                            auth_scopes=auth_scopes,
+                            auth_access_token=auth_access_token1,
+                            additional_request_headers=additional_request_headers,
+                            log_level=log_level,
+                            retry_count=retry_count,
+                            exclude_status_codes_from_retry=exclude_status_codes_from_retry,
+                        )
+                        if result:
+                            auth_access_token1 = result.access_token
+                            if result.request_id:
+                                request_id_list.append(result.request_id)
+                            responses.extend(result.responses)
+            else:
+                # send a whole batch to the server at once
+                json_data_list_1 = [
+                    (
+                        convert_dict_to_fhir_json(item.asDict(recursive=True))
+                        if "id" in item
+                        else item["value"]
+                    )
+                    for item in json_data_list
+                ]
+                first_item = (
+                    json.loads(json_data_list_1[0])
+                    if len(json_data_list_1) > 0
+                    else None
+                )
+                first_id = first_item.get("id") if first_item is not None else None
+                if first_id:
+                    result = send_json_bundle_to_fhir(
+                        id_=first_id,
+                        json_data_list=json_data_list_1,
                         server_url=server_url,
                         validation_server_url=validation_server_url,
                         resource=resource_name,
-                        logger=logger,
                         auth_server_url=auth_server_url,
                         auth_client_id=auth_client_id,
                         auth_client_secret=auth_client_secret,
                         auth_login_token=auth_login_token,
                         auth_scopes=auth_scopes,
-                        auth_access_token=auth_access_token1,
+                        auth_access_token=auth_access_token,
+                        additional_request_headers=additional_request_headers,
+                        logger=logger,
                         log_level=log_level,
                         retry_count=retry_count,
                         exclude_status_codes_from_retry=exclude_status_codes_from_retry,
                     )
+                    if result and result.request_id:
+                        request_id_list.append(result.request_id)
                     if result:
-                        auth_access_token1 = result.access_token
-                        if result.request_id:
-                            request_id_list.append(result.request_id)
-                        responses.extend(result.responses)
-            else:
-                # send a whole batch to the server at once
-                result = send_json_bundle_to_fhir(
-                    json_data_list=[
-                        convert_dict_to_fhir_json(item.asDict(recursive=True))
-                        if "id" in item
-                        else item["value"]
-                        for item in json_data_list
-                    ],
-                    server_url=server_url,
-                    validation_server_url=validation_server_url,
-                    resource=resource_name,
-                    auth_server_url=auth_server_url,
-                    auth_client_id=auth_client_id,
-                    auth_client_secret=auth_client_secret,
-                    auth_login_token=auth_login_token,
-                    auth_scopes=auth_scopes,
-                    auth_access_token=auth_access_token,
-                    logger=logger,
-                    log_level=log_level,
-                    retry_count=retry_count,
-                    exclude_status_codes_from_retry=exclude_status_codes_from_retry,
-                )
-                if result and result.request_id:
-                    request_id_list.append(result.request_id)
-                if result:
-                    responses = result.responses
+                        responses = result.responses
         # each item in responses is either a json object
         #   or a list of json objects
         error_count: int = 0
