@@ -130,9 +130,9 @@ class FrameworkFixedWidthLoader(FrameworkTransformer):
 
     def _transform(self, df: DataFrame) -> DataFrame:
         view = self.getView()
-        file_path: Union[
-            Path, str, Callable[[Optional[str]], Union[Path, str]]
-        ] = self.getFilePath()
+        file_path: Union[Path, str, Callable[[Optional[str]], Union[Path, str]]] = (
+            self.getFilePath()
+        )
         if callable(file_path):
             file_path = file_path(self.loop_id)
         columns: List[ColumnSpec] = self.getColumns()
@@ -144,7 +144,7 @@ class FrameworkFixedWidthLoader(FrameworkTransformer):
             progress_logger.write_to_log(f"Loading file for view {view}: {paths}")
             progress_logger.log_param("data_path", ", ".join(paths))
 
-        df_reader: DataFrameReader = df.sql_ctx.read
+        df_reader: DataFrameReader = df.sparkSession.read
         df_text = df_reader.text(paths=paths)
         if has_header:
             first: Optional[Row] = df_text.first()
