@@ -93,9 +93,9 @@ class FrameworkParquetLoader(FrameworkTransformer):
 
     def _transform(self, df: DataFrame) -> DataFrame:
         view: str = self.getView()
-        file_path: Union[
-            Path, str, Callable[[Optional[str]], Union[Path, str]]
-        ] = self.getFilePath()
+        file_path: Union[Path, str, Callable[[Optional[str]], Union[Path, str]]] = (
+            self.getFilePath()
+        )
         if callable(file_path):
             file_path = file_path(self.loop_id)
         name: Optional[str] = self.getName()
@@ -119,7 +119,7 @@ class FrameworkParquetLoader(FrameworkTransformer):
         ):
             try:
                 df_reader: Union[DataFrameReader, DataStreamReader] = (
-                    df.sql_ctx.read if not stream else df.sql_ctx.readStream
+                    df.sparkSession.read if not stream else df.sparkSession.readStream
                 )
 
                 mode = self.getMode()
