@@ -1,5 +1,6 @@
 import json
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional, Union
+
 
 from spark_pipeline_framework.transformers.athena_table_creator.v1.athena_table_creator import (
     AthenaTableCreator,
@@ -21,12 +22,19 @@ from pyspark.sql.dataframe import DataFrame
 from spark_pipeline_framework.logger.yarn_logger import get_logger
 from spark_pipeline_framework.progress_logger.progress_logger import ProgressLogger
 from spark_pipeline_framework.proxy_generator.proxy_base import ProxyBase
+from spark_pipeline_framework.utilities.fhir_helpers.fhir_sender_operation import (
+    FhirSenderOperation,
+)
 from spark_pipeline_framework.transformers.framework_mapping_runner.v1.framework_mapping_runner import (
     FrameworkMappingLoader,
 )
 from spark_pipeline_framework.transformers.framework_transformer.v1.framework_transformer import (
     FrameworkTransformer,
 )
+from spark_pipeline_framework.transformers.framework_parquet_exporter.v1.framework_parquet_exporter import (
+    FrameworkParquetExporter,
+)
+
 from spark_pipeline_framework.utilities.file_modes import FileWriteModes
 from spark_pipeline_framework.utilities.spark_data_frame_helpers import (
     spark_is_data_frame_empty,
@@ -44,6 +52,7 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         func_get_response_path: Callable[[str, str], str],
         fhir_server_url: str,
         source_entity_name: str,
+        file_format: str = "json",
         fhir_validation_url: Optional[str] = None,
         athena_schema: Optional[str] = None,
         name: Optional[str] = None,
@@ -56,6 +65,14 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         progress_logger: Optional[ProgressLogger] = None,
         send_to_fhir: Optional[bool] = True,
         mode: str = FileWriteModes.MODE_ERROR,
+        run_synchronously: Optional[bool] = None,
+        additional_request_headers: Optional[Dict[str, str]] = None,
+        sort_data: Optional[Dict[str, Dict[str, Any]]] = None,
+        partition_by_column_name: Optional[str] = None,
+        enable_repartitioning: bool = True,
+        operation: Union[
+            FhirSenderOperation, str
+        ] = FhirSenderOperation.FHIR_OPERATION_MERGE.value,
     ):
         """
         Runs the auto-mappers, saves the result to Athena db and then sends the results to fhir server
@@ -67,6 +84,19 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         :param fhir_validation_url: url to FHIR validation server
         :param source_entity_name: resource name
         :param athena_schema: schema to use when writing data to Athena
+        :param run_synchronously: (Optional) Run on the Spark master to make debugging easier on dev machines
+        :param additional_request_headers: (Optional) Additional request headers to use
+                                            (Eg: {"Accept-Charset": "utf-8"})
+        :param sort_data: (Optional) Whether to sort the data. Format - {
+                            KEY - view: view name,
+                            VALUE - {
+                                sort_by_column_name_and_type: "columnName and columnType to be used for sorting",
+                                drop_fields_from_json: (list) List of fields to drop from json,
+                                partition_by_column_name: (str) Name of the column that will be used to repartition df
+                            }
+                        }
+        :param enable_repartitioning: Enable repartitioning or not, default True
+        :param operation: The API operation to perform, such as merge, put, delete etc
         """
         super().__init__(
             name=name, parameters=parameters, progress_logger=progress_logger
@@ -96,6 +126,9 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         self.fhir_server_url: Param[str] = Param(self, "fhir_server_url", "")
         self._setDefault(fhir_server_url=fhir_server_url)
 
+        self.file_format: Param[str] = Param(self, "file_format", "")
+        self._setDefault(file_format=file_format)
+
         self.fhir_validation_url: Param[Optional[str]] = Param(
             self, "fhir_validation_url", ""
         )
@@ -132,21 +165,56 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         self.mode: Param[str] = Param(self, "mode", "")
         self._setDefault(mode=mode)
 
+        self.run_synchronously: Param[Optional[bool]] = Param(
+            self, "run_synchronously", ""
+        )
+        self._setDefault(run_synchronously=run_synchronously)
+
+        self.additional_request_headers: Param[Optional[Dict[str, str]]] = Param(
+            self, "additional_request_headers", ""
+        )
+        self._setDefault(additional_request_headers=additional_request_headers)
+
+        self.sort_data: Param[Optional[Dict[str, Dict[str, Any]]]] = Param(
+            self, "sort_data", ""
+        )
+        self._setDefault(sort_data=sort_data)
+
+        self.enable_repartitioning: Param[bool] = Param(
+            self, "enable_repartitioning", ""
+        )
+        self._setDefault(enable_repartitioning=enable_repartitioning)
+
+        self.operation: Param[Union[FhirSenderOperation, str]] = Param(
+            self, "operation", ""
+        )
+        self._setDefault(operation=operation)
+
         kwargs = self._input_kwargs
         self.setParams(**kwargs)
 
     def _transform(self, df: DataFrame) -> DataFrame:
         transformer: ProxyBase = self.getTransformer()
         func_get_path: Callable[[str, str], str] = self.getFuncGetPath()
-        func_get_response_path: Callable[
-            [str, str], str
-        ] = self.getFuncGetResponsePath()
+        func_get_response_path: Callable[[str, str], str] = (
+            self.getFuncGetResponsePath()
+        )
+        file_format: str = self.getFileFormat()
         fhir_server_url: str = self.getFhirServerUrl()
         fhir_validation_url: Optional[str] = self.getFhirValidationServerUrl()
         name: Optional[str] = self.getName()
         source_entity_name: str = self.getSourceEntityName()
         athena_schema: Optional[str] = self.getAthenaSchema()
         parameters = self.getParameters()
+        additional_request_headers: Optional[Dict[str, str]] = (
+            self.getAdditionalRequestHeaders()
+        )
+        sort_data: Optional[Dict[str, Dict[str, Any]]] = self.getOrDefault(
+            self.sort_data
+        )
+        enable_repartitioning: bool = self.getOrDefault(self.enable_repartitioning)
+        operation: Union[FhirSenderOperation, str] = self.getOrDefault(self.operation)
+
         assert parameters
         progress_logger = self.getProgressLogger()
 
@@ -158,6 +226,7 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         send_to_fhir: Optional[bool] = self.getSendToFhir()
 
         mode: str = self.getOrDefault(self.mode)
+        run_synchronously: Optional[bool] = self.getOrDefault(self.run_synchronously)
 
         self.logger.info(f"Calling {fhir_server_url} with client_id={auth_client_id}")
 
@@ -169,6 +238,17 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
         t: Transformer
         for t in transformer.transformers:
             if isinstance(t, FrameworkMappingLoader):
+                if hasattr(t, "getName"):
+                    # noinspection Mypy
+                    stage_name = t.getName()
+                    if not stage_name:
+                        stage_name = t.__class__.__name__
+                else:
+                    stage_name = t.__class__.__name__
+                if progress_logger is not None:
+                    progress_logger.start_mlflow_run(
+                        run_name=stage_name, is_nested=True
+                    )
                 t.transform(df)  # run the automapper
                 views: List[str] = t.getViews()
                 view: str
@@ -176,8 +256,16 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
                     self.logger.info(
                         f"---- Started processing for view: {view} --------"
                     )
+                    if progress_logger is not None:
+                        progress_logger.start_mlflow_run(
+                            run_name=f"Exporting view {view}", is_nested=True
+                        )
                     # get resource name
-                    result_df: DataFrame = df.sql_ctx.table(view)
+                    result_df: DataFrame = df.sparkSession.table(view)
+                    # True if sort_data field exists and view name is present as dict key
+                    need_sorting: bool = (
+                        True if (sort_data and sort_data.get(view)) else False
+                    )
                     if spark_is_data_frame_empty(df=result_df):
                         self.logger.info(f"No data to export/send for view '{view}'")
                         continue
@@ -190,15 +278,27 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
                     fhir_resource_response_path: str = func_get_response_path(
                         view, resourceType
                     )
+
                     # export as FHIR to local fhir_resource_path
-                    FhirExporter(
-                        view=view,
-                        file_path=fhir_resource_path,
-                        parameters=parameters,
-                        progress_logger=progress_logger,
-                        name=f"{name}_fhir_exporter",
-                        mode=mode,
-                    ).transform(df)
+                    if file_format == "parquet":
+                        FrameworkParquetExporter(
+                            name=f"{name}_parquet_exporter",
+                            parameters=parameters,
+                            progress_logger=progress_logger,
+                            view=view,
+                            mode=mode,
+                            file_path=fhir_resource_path,
+                        ).transform(df)
+                    else:
+                        FhirExporter(
+                            view=view,
+                            file_path=fhir_resource_path,
+                            parameters=parameters,
+                            progress_logger=progress_logger,
+                            name=f"{name}_fhir_exporter",
+                            mode=mode,
+                        ).transform(df)
+
                     if fhir_resource_path.startswith("s3") and athena_schema:
                         # create table in Athena
                         s3_temp_folder: str = str(parameters.get("s3_temp_folder"))
@@ -219,6 +319,7 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
                             server_url=fhir_server_url,
                             validation_server_url=fhir_validation_url,
                             file_path=fhir_resource_path,
+                            file_format=file_format,
                             response_path=fhir_resource_response_path,
                             parameters=parameters,
                             progress_logger=progress_logger,
@@ -232,8 +333,32 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
                             auth_login_token=auth_login_token,
                             auth_scopes=auth_scopes,
                             name=f"{name}_fhir_sender",
+                            additional_request_headers=additional_request_headers,
                             mode=mode,
+                            run_synchronously=run_synchronously,
+                            num_partitions=parameters.get("num_partitions"),
+                            sort_by_column_name_and_type=(
+                                sort_data[view].get("sort_by_column_name_and_type")
+                                if need_sorting and sort_data
+                                else None
+                            ),
+                            drop_fields_from_json=(
+                                sort_data[view].get("drop_fields_from_json")
+                                if need_sorting and sort_data
+                                else None
+                            ),
+                            partition_by_column_name=(
+                                sort_data[view].get("partition_by_column_name")
+                                if need_sorting and sort_data
+                                else None
+                            ),
+                            enable_repartitioning=enable_repartitioning,
+                            operation=operation,
                         ).transform(df)
+                    if progress_logger is not None:
+                        progress_logger.end_mlflow_run()
+                if progress_logger is not None:
+                    progress_logger.end_mlflow_run()
         return df
 
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
@@ -252,6 +377,10 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
     def getFhirServerUrl(self) -> str:
         return self.getOrDefault(self.fhir_server_url)
 
+    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
+    def getFileFormat(self) -> str:
+        return self.getOrDefault(self.file_format)
+
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
     def getFhirValidationServerUrl(self) -> Optional[str]:
         return self.getOrDefault(self.fhir_validation_url)
@@ -287,3 +416,7 @@ class AutoMapperToFhirTransformer(FrameworkTransformer):
     # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
     def getSendToFhir(self) -> Optional[bool]:
         return self.getOrDefault(self.send_to_fhir)
+
+    # noinspection PyPep8Naming,PyMissingOrEmptyDocstring
+    def getAdditionalRequestHeaders(self) -> Optional[Dict[str, str]]:
+        return self.getOrDefault(self.additional_request_headers)
