@@ -24,6 +24,7 @@ from helix_fhir_client_sdk.responses.fhir_get_response import FhirGetResponse
 from pyspark.sql.types import (
     Row,
 )
+from helix_fhir_client_sdk.function_types import RefreshTokenFunction
 
 from spark_pipeline_framework.logger.yarn_logger import get_logger
 from spark_pipeline_framework.utilities.fhir_helpers.fhir_get_response_writer import (
@@ -73,6 +74,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -84,30 +86,71 @@ class FhirReceiverHelpers:
         url_column: Optional[str],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction] = None,
     ) -> List[Row]:
         """
-        This function processes a partition
+        This function processes a partition calling fhir server for each row in the partition
 
         This has to be a static function to avoid creating a closure around a class
         https://spark.apache.org/docs/latest/rdd-programming-guide.html#passing-functions-to-spark
 
 
+        :param partition_index: partition index
+        :param rows: rows to process
+        :param batch_size: batch size
+        :param has_token_col: has token column
+        :param server_url: server url
+        :param log_level: log level
+        :param action: action to perform
+        :param action_payload: action payload
+        :param additional_parameters: additional parameters
+        :param filter_by_resource: filter by resource
+        :param filter_parameter: filter parameter
+        :param sort_fields: sort fields
+        :param auth_server_url: auth server url
+        :param auth_client_id: auth client id
+        :param auth_client_secret: auth client secret
+        :param auth_login_token: auth login token
+        :param auth_scopes: auth scopes
+        :param include_only_properties: include only properties
+        :param separate_bundle_resources: separate bundle resources
+        :param expand_fhir_bundle: expand fhir bundle
+        :param accept_type: accept type
+        :param content_type: content type
+        :param additional_request_headers: Additional request headers
+        :param accept_encoding: accept encoding
+        :param slug_column: slug column
+        :param retry_count: retry count
+        :param exclude_status_codes_from_retry: exclude status codes from retry
+        :param limit: limit
+        :param auth_access_token: auth access token
+        :param resource_type: resource type
+        :param error_view: error view
+        :param url_column: url column
+        :param use_data_streaming: use data streaming
+        :param graph_json: graph json
+        :param refresh_token_function: function to refresh token
+        :return: rows
         """
         resource_id_with_token_list: List[Dict[str, Optional[str]]] = [
-            {
-                "resource_id": r["id"],
-                "access_token": r["token"],
-                url_column: r[url_column],
-                slug_column: r[slug_column],
-                "resourceType": r["resourceType"],
-            }
-            if has_token_col and url_column and slug_column
-            else {
-                "resource_id": r["id"],
-                "access_token": r["token"],
-            }
-            if has_token_col
-            else {"resource_id": r["id"], "access_token": auth_access_token}
+            (
+                {
+                    "resource_id": r["id"],
+                    "access_token": r["token"],
+                    url_column: r[url_column],
+                    slug_column: r[slug_column],
+                    "resourceType": r["resourceType"],
+                }
+                if has_token_col and url_column and slug_column
+                else (
+                    {
+                        "resource_id": r["id"],
+                        "access_token": r["token"],
+                    }
+                    if has_token_col
+                    else {"resource_id": r["id"], "access_token": auth_access_token}
+                )
+            )
             for r in rows
         ]
         result = FhirReceiverHelpers.process_with_token(
@@ -133,6 +176,7 @@ class FhirReceiverHelpers:
             expand_fhir_bundle=expand_fhir_bundle,
             accept_type=accept_type,
             content_type=content_type,
+            additional_request_headers=additional_request_headers,
             accept_encoding=accept_encoding,
             slug_column=slug_column,
             retry_count=retry_count,
@@ -144,6 +188,7 @@ class FhirReceiverHelpers:
             url_column=url_column,
             use_data_streaming=use_data_streaming,
             graph_json=graph_json,
+            refresh_token_function=refresh_token_function,
         )
         return result
 
@@ -172,6 +217,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -183,6 +229,7 @@ class FhirReceiverHelpers:
         url_column: Optional[str],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> List[Row]:
         try:
             first_id: Optional[str] = resource_id_with_token_list[0]["resource_id"]
@@ -239,6 +286,7 @@ class FhirReceiverHelpers:
                 expand_fhir_bundle=expand_fhir_bundle,
                 accept_type=accept_type,
                 content_type=content_type,
+                additional_request_headers=additional_request_headers,
                 accept_encoding=accept_encoding,
                 slug_column=slug_column,
                 retry_count=retry_count,
@@ -249,6 +297,7 @@ class FhirReceiverHelpers:
                 error_view=error_view,
                 use_data_streaming=use_data_streaming,
                 graph_json=graph_json,
+                refresh_token_function=refresh_token_function,
             )
         else:  # otherwise send one by one
             return FhirReceiverHelpers.process_one_by_one(
@@ -274,6 +323,7 @@ class FhirReceiverHelpers:
                 expand_fhir_bundle=expand_fhir_bundle,
                 accept_type=accept_type,
                 content_type=content_type,
+                additional_request_headers=additional_request_headers,
                 accept_encoding=accept_encoding,
                 slug_column=slug_column,
                 retry_count=retry_count,
@@ -284,6 +334,7 @@ class FhirReceiverHelpers:
                 resource_name=resource_type,
                 use_data_streaming=use_data_streaming,
                 graph_json=graph_json,
+                refresh_token_function=refresh_token_function,
             )
 
     @staticmethod
@@ -311,6 +362,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -321,6 +373,7 @@ class FhirReceiverHelpers:
         resource_name: str,
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> List[Row]:
         coroutines: List[Coroutine[Any, Any, List[Row]]] = [
             FhirReceiverHelpers.process_single_row_async(
@@ -345,6 +398,7 @@ class FhirReceiverHelpers:
                 expand_fhir_bundle=expand_fhir_bundle,
                 accept_type=accept_type,
                 content_type=content_type,
+                additional_request_headers=additional_request_headers,
                 accept_encoding=accept_encoding,
                 slug_column=slug_column,
                 retry_count=retry_count,
@@ -356,6 +410,7 @@ class FhirReceiverHelpers:
                 resource_name=resource_name,
                 use_data_streaming=use_data_streaming,
                 graph_json=graph_json,
+                refresh_token_function=refresh_token_function,
             )
             for resource1 in resource_id_with_token_list
         ]
@@ -391,6 +446,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -402,6 +458,7 @@ class FhirReceiverHelpers:
         resource1: Dict[str, Optional[str]],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> List[Row]:
         id_ = resource1["resource_id"]
         access_token = resource1["access_token"]
@@ -437,6 +494,7 @@ class FhirReceiverHelpers:
                     expand_fhir_bundle=expand_fhir_bundle,
                     accept_type=accept_type,
                     content_type=content_type,
+                    additional_request_headers=additional_request_headers,
                     accept_encoding=accept_encoding,
                     slug_column=slug_column,
                     retry_count=retry_count,
@@ -444,6 +502,7 @@ class FhirReceiverHelpers:
                     limit=limit,
                     use_data_streaming=use_data_streaming,
                     graph_json=graph_json,
+                    refresh_token_function=refresh_token_function,
                 )
             )
             resp_result: str = result1.responses.replace("\n", "")
@@ -516,6 +575,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -526,6 +586,7 @@ class FhirReceiverHelpers:
         error_view: Optional[str],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> List[Row]:
         result1 = asyncio.run(
             FhirReceiverHelpers.send_simple_fhir_request_async(
@@ -551,6 +612,7 @@ class FhirReceiverHelpers:
                 expand_fhir_bundle=expand_fhir_bundle,
                 accept_type=accept_type,
                 content_type=content_type,
+                additional_request_headers=additional_request_headers,
                 accept_encoding=accept_encoding,
                 slug_column=slug_column,
                 retry_count=retry_count,
@@ -558,6 +620,7 @@ class FhirReceiverHelpers:
                 limit=limit,
                 use_data_streaming=use_data_streaming,
                 graph_json=graph_json,
+                refresh_token_function=refresh_token_function,
             )
         )
         resp_result: str = result1.responses.replace("\n", "")
@@ -624,6 +687,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         slug_column: Optional[str],
         retry_count: Optional[int],
@@ -631,6 +695,7 @@ class FhirReceiverHelpers:
         limit: Optional[int],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> FhirGetResponse:
         url = server_url_ or server_url
         assert url
@@ -656,16 +721,18 @@ class FhirReceiverHelpers:
             expand_fhir_bundle=expand_fhir_bundle,
             accept_type=accept_type,
             content_type=content_type,
+            additional_request_headers=additional_request_headers,
             accept_encoding=accept_encoding,
-            extra_context_to_return={slug_column: service_slug}
-            if slug_column and service_slug
-            else None,
+            extra_context_to_return=(
+                {slug_column: service_slug} if slug_column and service_slug else None
+            ),
             retry_count=retry_count,
             exclude_status_codes_from_retry=exclude_status_codes_from_retry,
             limit=limit,
             log_level=log_level,
             use_data_streaming=use_data_streaming,
             graph_json=graph_json,
+            refresh_token_function=refresh_token_function,
         )
 
     @staticmethod
@@ -697,6 +764,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool = True,
         accept_type: Optional[str] = None,
         content_type: Optional[str] = None,
+        additional_request_headers: Optional[Dict[str, str]] = None,
         accept_encoding: Optional[str] = None,
         extra_context_to_return: Optional[Dict[str, Any]] = None,
         retry_count: Optional[int] = None,
@@ -704,6 +772,7 @@ class FhirReceiverHelpers:
         limit: Optional[int] = None,
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> FhirGetResponse:
         """
         Sends a fhir request to the fhir client sdk
@@ -735,6 +804,7 @@ class FhirReceiverHelpers:
         :param expand_fhir_bundle: expands the fhir bundle to create a list of resources
         :param accept_type: (Optional) Accept header to use
         :param content_type: (Optional) Content-Type header to use
+        :param additional_request_headers: (Optional) Additional request headers to add in FHIR request
         :param accept_encoding: (Optional) Accept-encoding header to use
         :param extra_context_to_return: a dict to return with every row (separate_bundle_resources is set)
                                         or with FhirGetResponse
@@ -744,6 +814,7 @@ class FhirReceiverHelpers:
         :param log_level:
         :param use_data_streaming:
         :param graph_json:
+        :param refresh_token_function: function to refresh token
         :return:
         :rtype:
         """
@@ -770,6 +841,13 @@ class FhirReceiverHelpers:
             fhir_client = fhir_client.content_type(content_type)
         if accept_encoding is not None:
             fhir_client = fhir_client.accept_encoding(accept_encoding)
+        if additional_request_headers is not None:
+            logger.info(
+                f"Additional Request Headers to be sent - {additional_request_headers}"
+            )
+            fhir_client = fhir_client.additional_request_headers(
+                additional_request_headers
+            )
 
         fhir_client = fhir_client.resource(resource_name)
         if resource_id:
@@ -819,11 +897,16 @@ class FhirReceiverHelpers:
         if use_data_streaming:
             fhir_client = fhir_client.use_data_streaming(use_data_streaming)
 
+        if refresh_token_function:
+            fhir_client = fhir_client.refresh_token_function(refresh_token_function)
+
         return (
             await fhir_client.simulate_graph_async(
-                id_=cast(str, resource_id)
-                if not isinstance(resource_id, list)
-                else resource_id[0],
+                id_=(
+                    cast(str, resource_id)
+                    if not isinstance(resource_id, list)
+                    else resource_id[0]
+                ),
                 graph_json=graph_json,
                 contained=False,
                 separate_bundle_resources=separate_bundle_resources,
@@ -871,6 +954,7 @@ class FhirReceiverHelpers:
         expand_fhir_bundle: bool,
         accept_type: Optional[str],
         content_type: Optional[str],
+        additional_request_headers: Optional[Dict[str, str]],
         accept_encoding: Optional[str],
         retry_count: Optional[int],
         exclude_status_codes_from_retry: Optional[List[int]],
@@ -879,6 +963,7 @@ class FhirReceiverHelpers:
         ignore_status_codes: List[int],
         use_data_streaming: Optional[bool],
         graph_json: Optional[Dict[str, Any]],
+        refresh_token_function: Optional[RefreshTokenFunction],
     ) -> GetBatchResult:
         resources: List[str] = []
         errors: List[str] = []
@@ -915,12 +1000,14 @@ class FhirReceiverHelpers:
                     expand_fhir_bundle=expand_fhir_bundle,
                     accept_type=accept_type,
                     content_type=content_type,
+                    additional_request_headers=additional_request_headers,
                     accept_encoding=accept_encoding,
                     retry_count=retry_count,
                     exclude_status_codes_from_retry=exclude_status_codes_from_retry,
                     log_level=log_level,
                     use_data_streaming=use_data_streaming,
                     graph_json=graph_json,
+                    refresh_token_function=refresh_token_function,
                 )
             )
             try:
@@ -974,12 +1061,14 @@ class FhirReceiverHelpers:
                         expand_fhir_bundle=expand_fhir_bundle,
                         accept_type=accept_type,
                         content_type=content_type,
+                        additional_request_headers=additional_request_headers,
                         accept_encoding=accept_encoding,
                         retry_count=retry_count,
                         exclude_status_codes_from_retry=exclude_status_codes_from_retry,
                         log_level=log_level,
                         use_data_streaming=use_data_streaming,
                         graph_json=graph_json,
+                        refresh_token_function=refresh_token_function,
                     )
                 )
                 result_response: List[str] = []
