@@ -3,6 +3,8 @@ from typing import Optional, List
 
 from helix_fhir_client_sdk.fhir_client import FhirClient
 
+from spark_pipeline_framework.utilities.fhir_helpers.token_helper import TokenHelper
+
 
 def get_fhir_client(
     logger: Logger,
@@ -14,6 +16,7 @@ def get_fhir_client(
     auth_access_token: Optional[str] = None,
     auth_scopes: Optional[List[str]] = None,
     log_level: Optional[str] = None,
+    auth_well_known_url: Optional[str] = None,
 ) -> FhirClient:
     assert server_url
 
@@ -25,6 +28,14 @@ def get_fhir_client(
 
     if auth_server_url:
         fhir_client = fhir_client.auth_server_url(auth_server_url)
+    if auth_well_known_url:
+        auth_server_url_from_wellknown: Optional[str] = (
+            TokenHelper.get_auth_server_url_from_well_known_url(
+                well_known_url=auth_well_known_url
+            )
+        )
+        if auth_server_url_from_wellknown:
+            fhir_client = fhir_client.auth_server_url(auth_server_url_from_wellknown)
     if auth_client_id and auth_client_secret:
         fhir_client = fhir_client.client_credentials(auth_client_id, auth_client_secret)
     if auth_login_token:
