@@ -10,7 +10,7 @@ from spark_pipeline_framework.utilities.spark_data_frame_helpers import (
 def nonzero_view_rowcount(view: str) -> Callable[[DataFrame], bool]:
     def inner(df: DataFrame) -> bool:
         try:
-            return not spark_is_data_frame_empty(df.sql_ctx.table(view))
+            return not spark_is_data_frame_empty(df.sparkSession.table(view))
         except AnalysisException:
             return False
 
@@ -20,7 +20,10 @@ def nonzero_view_rowcount(view: str) -> Callable[[DataFrame], bool]:
 def same_rowcount_two_views(view1: str, view2: str) -> Callable[[DataFrame], bool]:
     def inner(df: DataFrame) -> bool:
         try:
-            return df.sql_ctx.table(view1).count() == df.sql_ctx.table(view2).count()
+            return (
+                df.sparkSession.table(view1).count()
+                == df.sparkSession.table(view2).count()
+            )
         except AnalysisException:
             return False
 
